//////////////////////////////////////
//           Wind Earth             //
//                                  //
//        by Pascal Chirol,         //
//       www.UpdatePixels.net       //
//////////////////////////////////////
import processing.opengl.*;
import hypermedia.net.*;
//////////////////////////////////////
UDP udp;
String ip = "localhost";
///////////////////
PFont fontLB;
PFont fontC;
PImage logo;
///////////////////
String[] metarList;
String[] texte;
String[] info = new String [15];
int nbline = 0;
int count = 0;
int nbfailure = 0;
boolean intro = true;
boolean start = false;
boolean runing = false;
boolean loading = false;
color couleur = color(0);
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
void setup(){
  size(350,400);
  noLoop();
  smooth();
  /////////////////
  udp = new UDP(this, 7000);
  udp.log(true);
  udp.listen(true);
  /////////////////
  fontLB = loadFont("LevenimMTBold-48.vlw");
  fontC = loadFont("CourierNewPS-BoldMT-48.vlw");
  /////////////////
  metarList = loadStrings("data/True_Metar.txt");
  logo = loadImage("data/logo.png");
  texte = loadStrings("texte.txt");
  draw();
}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void draw(){
  gui();
}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void receive(byte[] dataR, String ip, int port ) {
  try{
    String message = new String(dataR);
    ////////
    if(message.equals("alive?")){
      couleur = color(0,200,50);
      udp.send("alive!", ip, 6000 );
      texteGUI("Wind Eath GUI is Properly Runnig !");
      runing = true;
    }
    else if(message.equals("exit")){
      couleur = color(255);
      texteGUI("Wind Eath closed !");
      runing = false;
      start = false;
    }
    else{
      if(loading == false){
        int i = int(message);
        loadingDATA(i);
      }
    }
  }  
  catch(Exception x){
    texteGUI(""+x);
  }
}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void loadingDATA(int i){
  if(start){
    if(loading == false){
      loading = true;
      texteGUI("Loading DATA for "+metarList[i]);
      /////////////////////////
      try {
        URL url = new URL("ftp://tgftp.nws.noaa.gov/data/observations/metar/decoded/"+metarList[i]+".TXT");
        String s;
        String[] data = new String[0];
        BufferedReader buffer = new BufferedReader( new InputStreamReader(url.openStream()) );
        while( ( s=buffer.readLine())!=null ) { // pour chaque lignes
          data = append(data, s);
        }
        buffer.close();
        if(data != null){
          runing = true;
          saveStrings("data/metar/"+metarList[i]+".txt", data);
          udp.send(str (i), ip, 6000 );
          texteGUI("  + DATA loaded");
          nbfailure = 0;
        }
      }
      catch( MalformedURLException e ) {
        println(nbfailure);
        nbfailure++;
        texteGUI("Network Disconnected !");
        texteGUI("Please Look you Network Connection !");
        texteGUI("  + Loading Failded for "+metarList[i]);
        texteGUI("  + "+e);
        if(nbfailure<10)
          loadingDATA(i);
      }
      catch( IOException e) {
        println(nbfailure);
        nbfailure++;
        texteGUI("Network Disconnected !");
        texteGUI("Please Look you Network Connection !");
        texteGUI("  + Loading Failded for "+metarList[i]);
        texteGUI("  + "+e);
        if(nbfailure<10)
          loadingDATA(i);
        //if ( e instanceof FileNotFoundException ) ;    // ... 404
        //if ( e instanceof SocketException ) ;          // connection reset
        //if ( e instanceof NoRouteToHostException ) ;   // connexion inexistante
        //if ( e instanceof UnknownHostException ) ;     // serveur introuvable
      }
      finally {
        loading = false;
      }
      ///////////////////////////////////////////////////////////////////////////
    }
  }
}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void launching(){
  if(runing == false){
    if(nbfailure<10){
      int i = int(random(metarList.length));
      loadingDATA(i);
      launching();
      nbfailure = 0;
    }
    else{
      start = false;
      runing = false;
      loading = false;
      texteGUI("");
      texteGUI("RETRY TO LAUNCH SOFTWARE !");
      redraw();
    }
  }
  else{
    texteGUI("  + Network Connected...");
    texteGUI("Launching Wind Earth GUI!");
    open("data\\WindEarth_GUI.exe");//PC
    open("data/WindEarth_GUI.app");//MAC
    texteGUI("  + Waiting for Wind Earth GUI...");
  }
}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void mousePressed() {
  if((mouseX > 0) && (mouseX < width) && (mouseY < 100)) {
    if(start == false){ 
      couleur = color(255,150,0);
      texteGUI("Launching METAR Network Connector !");
      texteGUI("  + Waiting for METAR Network Connector...");
      //////////
      intro = false;
      start = true;
      redraw();
      //////////
      launching();
    }
  }
  if((mouseX > 0) && (mouseX < width) && (mouseY > height-50) && (mouseY < height)) {
    link("http://updatepixels.net", "_new");
  }
}
///////////////////////////////////////////////////////////////////////////
void mouseMoved() {
  if(start == false){ 
    if((mouseX > 0) && (mouseX < width) && (mouseY < 100)) {
      couleur = color(255);
    }
    else{
      couleur = color(0);
    }
  }
  redraw();
}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void texteGUI(String txt){
  if(nbline > info.length-1){
    for(int i=0;i<info.length-1;i++){
      info[i]=info[i+1];
    }
    nbline = info.length-1;
  }
  println(txt);
  info[nbline] = txt;
  nbline++;
  redraw();
}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void gui(){
  background(140);
  //image(map,0,60);
  image(logo,10,10,100,100);
  fill(255);
  textFont(fontLB, 35);
  text("WIND EARTH",130,60);
  /////
  textFont(fontLB, 11);
  text("meteorological airport report",130,30);
  /////
  stroke(255);
  //line(110,60,width-10,60);
  /////
  fill(couleur);
  ellipse(135,90,14,14);
  /////
  stroke(255);
  line(10,110,width-10,110);
  if(start == false){
    fill(255);
    textFont(fontLB, 14);
    text("Launch Software",150,95);
  }
  else{
    fill(255);
    textFont(fontLB, 14);
    text("Launching Software . . .",150,95);
  }
  if(intro){
    textFont(fontC, 13);
    for(int i=0;i<texte.length;i++){
      text(texte[i],10,130+(i*13));
    }
  }
  else{
    textFont(fontC, 13);
    for(int i=0;i<info.length;i++){
      if(info[i] != null)
        text(info[i],10,130+(i*13));
    }
  }
  /////
  line(10,height-50,width-10,height-50);
  textFont(fontLB, 12);
  text("(cc)   P a s c a l  C h i r o l ,     J u n e  2 0 0 7",10,height-35);
  text("h t t p : / / U p d a t e P i x e l s . n e t",10,height-20);
}
